package de.hft.stuttgart.citydoctor2.healing;

import java.io.IOException;
import java.util.List;

import de.hft.stuttgart.citydoctor2.check.ModificationListener;
import de.hft.stuttgart.citydoctor2.check.ValidationConfiguration;
import de.hft.stuttgart.citydoctor2.datastructure.CityDoctorModel;
import de.hft.stuttgart.citydoctor2.datastructure.CityObject;
import de.hft.stuttgart.citydoctor2.datastructure.Geometry;
import de.hft.stuttgart.citydoctor2.datastructure.Polygon;
import de.hft.stuttgart.citydoctor2.parser.CityGmlParseException;
import de.hft.stuttgart.citydoctor2.parser.CityGmlParser;
import de.hft.stuttgart.citydoctor2.parser.InvalidGmlFileException;

public class TestUtil {
	
	public static ModificationListener createDummyModificationListener() {
		return new ModificationListener() {
			
			@Override
			public void polygonRemoved(Polygon p) {
				// nothing
			}
			
			@Override
			public void polygonAdded(Polygon p) {
				// nothing
			}
		};
	}

	public static CityDoctorModel loadCityModel(String path, ValidationConfiguration config) throws CityGmlParseException, IOException, InvalidGmlFileException {
		return CityGmlParser.parseCityGmlFile(path, config.getParserConfiguration());
	}
	
	public static Polygon getPolygonById(String gmlId, CityDoctorModel model) {
		if (gmlId == null) {
			return null;
		}
		Polygon result = getPolygonById(gmlId, model.getBuildings());
		if (result != null) {
			return result;
		}
		result = getPolygonById(gmlId, model.getVegetation());
		if (result != null) {
			return result;
		}
		return getPolygonById(gmlId, model.getTransportation());
	}

	private static Polygon getPolygonById(String gmlId, List<? extends CityObject> features) {
		if (gmlId == null) {
			return null;
		}
		for (CityObject co : features) {
			Polygon result = getPolygonById(gmlId, co);
			if (result != null) {
				return result;
			}
		}
		return null;
	}

	public static Polygon getPolygonById(String gmlId, String featureId, CityDoctorModel model) {
		if (gmlId == null || featureId == null) {
			return null;
		}
		CityObject co = getCityObjectById(featureId, model);
		return getPolygonById(gmlId, co);
	}

	private static Polygon getPolygonById(String gmlId, CityObject co) {
		for (Geometry geom : co.getGeometries()) {
			for (Polygon p : geom.getPolygons()) {
				if (gmlId.equals(p.getGmlId().getGmlString())) {
					return p;
				}
			}
		}
		return null;
	}

	private static CityObject getCityObjectById(String featureId, CityDoctorModel model) {
		if (featureId == null) {
			return null;
		}
		for (CityObject co : model.getBuildings()) {
			if (featureId.equals(co.getGmlId().getGmlString())) {
				return co;
			}
		}
		for (CityObject co : model.getTransportation()) {
			if (featureId.equals(co.getGmlId().getGmlString())) {
				return co;
			}
		}
		for (CityObject co : model.getVegetation()) {
			if (featureId.equals(co.getGmlId().getGmlString())) {
				return co;
			}
		}
		return null;
	}

}
